from pydantic import BaseModel


from .Works import Work
from .Subjects import Subject
from .Users import User
import datetime

class LESSON(BaseModel):
    id: int
    id_str: str = None
    title: str = None
    date: datetime.datetime = None
    number: int | str |  None = None
    subject: Subject = None
    group: int = None
    status: str = None
    resultPlaceId: int = None
    works: list[Work] = []
    teachers: list[int] = []
    teachers_str: list[str] = []

class Lesson(BaseModel):
    id: int
    title: str = None
    date: datetime.datetime = None
    number: int = None
    subjectId: int
    status: str = None
    resultPlaceId: int = None
    building: str = None
    place: str = None
    floor: str = None
    hours: str = None
    works: list[int] = []
    teachers: list[int] = []
    teachers_str: list[str] = []

class HomeWorkFile(BaseModel):
    id: int
    id_str: str
    name: str = None
    typeGroup: str = None
    type: str
    pageUrl: str = None
    downloadUrl: str
    user: User
    size: int = None
    vote: int = None
    uploadedDate: datetime.datetime = None
    storageType: str = None

class Teacher(BaseModel):
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str = None
    lastName: str = None
    middleName: str = None
    shortName: str = None
    sex: str = None

class HomeWork(BaseModel):
    """[GET] /v2.0/users/me/school/{school}/homeworks\n~~~\n/v2.0/users/me/school/homeworks\n~~~\n/v2.0/persons/{person}/school/{school}/homeworks\n~~~\nДомашние задания\n~~~\nПрава доступа: EducationalInfo\n~~~"""
    
    works: list[Work] = []
    subjects: list[Subject] = []
    lessons: list[Lesson] = []
    files: list[HomeWorkFile] = []
    teachers: list[Teacher] = []



from pydantic import BaseModel

class Subject(BaseModel):
    """[GET] /v2.0/edu-groups/{eduGroup}/subjects\n~~~\n/v2.0/schools/{school}/subjects\n~~~\nСписок предметов, преподаваемых в классе в текущем отчётном периоде или преподаваемых в образовательной организации в текущем учебном году\n~~~\nПрава доступа: EducationalInfo\n~~~"""
    
    id: int
    id_str: str = None
    name: str
    knowledgeArea: str = None
    fgosSubjectId: int = None



from datetime import datetime
from pydantic import BaseModel


class Mark(BaseModel):
    """Оценка"""
    
    
    id: int
    id_str: str
    type: str
    value: str
    textValue: str
    person: int
    person_str: str
    work: int
    work_str: str
    lesson: int
    lesson_str: str
    number: int
    date: datetime
    workType: int
    mood: str
    use_avg_calc: bool



from datetime import datetime
from pydantic import BaseModel

class _Student(BaseModel):
    """[GET] /v2.0/teacher/{teacher}/students\n~~~\nСписок учеников для учителя который ведет уроки у этих учеников(они должны быть в расписании) от недели назад и на 30 дней вперед (Данный класс - 1 ученик)\n~~~"""
    
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    sex: str


class TeacherStudent(BaseModel):
    id: int
    id_str: str
    students: list[_Student]


class SchoolTeacher(BaseModel):
    """[GET] /v2.0/schools/{school}/teachers\n~~~\nСписок преподавателей в выбранной образовательной организации (Данный класс - 1 учитель)\n~~~"""
    
    Id: int
    IdStr: str
    UserId: int
    UserIdStr: str
    FirstName: str
    LastName: str
    MiddleName: str
    ShortName: str
    Sex: str
    DateBirth: datetime
    Email: str
    Subjects: str
    HouseMaster: bool
    Education: str
    ScientificDegree: str
    StartDate: datetime
    PedagogicalActivityDate: datetime
    ManagingEmployee: bool
    NameManagingPosition: str
    TeachingStaff: bool
    NameTeacherPosition: str
    TrainingAndSupportStaff: bool
    ServicePersonnel: bool
    MedicalWorker: bool
    NameMedicalPosition: str
    ExternalPartTime: bool


class EduGroupTeacher(BaseModel):
    """[GET] /v2.0/edu-groups/{group}/teachers\n~~~\nСписок учителей, которые ведут уроки в данной группе, учитываются уроки от недели назад и на 30 дней вперед (Данный класс - 1 учитель)\n~~~"""
    
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    sex: str




from datetime import datetime
from pydantic import BaseModel

class LessonLogEntries(BaseModel):
    """Отметка о посещаемости (Данный класс - 1 отметка)\n~~~"""
    
    person: int = None
    lesson: int = None
    person_str: str = None
    lesson_str: str = None
    comment: str = None
    status: str = None
    createdDate: datetime = None



from datetime import datetime
from pydantic import BaseModel

from .Works import Work
from .Subjects import Subject


class ScheduleDayLesson(BaseModel):
    id: int
    title: str
    date: datetime
    number: int
    subjectId: int
    status: str
    resultPlaceId: int
    building: str
    place: str
    floor: str
    hours: str
    works: list[int]
    teachers: list[int]


class ScheduleDayMark(BaseModel):
    id: int
    id_str: str
    type: str
    value: str
    textValue: str
    person: int
    person_str: str
    work: int
    work_str: str
    lesson: int
    lesson_str: str
    number: int
    date: datetime
    workType: int
    mood: str
    use_avg_calc: bool


class ScheduleDayWorkType(BaseModel):
    id: int
    schoolId: int
    abbreviation: str
    name: str
    isFinal: bool
    isImportant: bool
    kindId: int
    kind: str


class ScheduleDayTeacherPerson(BaseModel):
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    sex: str


class ScheduleDayTeacher(BaseModel):
    person: ScheduleDayTeacherPerson
    role: str


class ScheduleDayLessonLogEntries(BaseModel):
    person: int
    lesson: int
    person_str: str
    lesson_str: str
    comment: str
    status: str
    createdDate: datetime


class ScheduleDay(BaseModel):
    date: datetime
    lessons: list[ScheduleDayLesson]
    marks: list[ScheduleDayMark]
    works: list[Work]
    homeworks: list[Work]
    subjects: list[Subject]
    workTypes: list[ScheduleDayWorkType]
    lessonLogEntries: list[ScheduleDayLessonLogEntries]
    teachers: list[ScheduleDayTeacher]
    nextDate: datetime


class Schedule(BaseModel):
    """[GET] /v2.0/persons/{person}/groups/{group}/schedules\n~~~\nРасписание ученика\n~~~\nПрава доступа: EducationalInfo\n~~~"""
    
    days: list[ScheduleDay]


from . import api, types, exceptions, asyncapi

METHODS = {
    "Authorities": [
        "/v2.0/users/me/organizations",
        "/v2.0/users/me/organizations/{organizationId}"],
    "Authorizations": ["/v2.0/authorizations"],
    "AverageMarks": [
        "/v2.0/persons/{person}/reporting-periods/{period}/avg-mark",
        "/v2.0/persons/{person}/reporting-periods/{period}/subjects/{subject}/avg-mark",
        "/v2.0/edu-groups/{group}/reporting-periods/{period}/avg-marks/{date}",
        "/v2.0/edu-groups/{group}/avg-marks/{from}/{to}"],
    "Children": [
        "/v2.0/user/{userID}/children",
        "/v2.0/person/{personID}/children"],
    "Classmates": ["/v2.0/users/me/classmates"],
    "Context": [
        "/v2.0/users/me/context",
        "/v2.0/users/{userId}/context"],
    "CriteriaJournalMarks": [""],
    "": [""], # TODO: дополнить всё с https://api.school.mosreg.ru/partners/swagger/ui/index#/
}



from pydantic import BaseModel

class WorkType(BaseModel):
    """[GET] /v2.0/work-types/{school}\n~~~\nПолучение списка всех типов работ школы (Данный класс - 1 тип работы)\n~~~\nПрава доступа: EducationalInfo\n~~~"""
    
    id: int
    id_str: str
    schoolId: int = None
    kindId: int = None
    kind: str = None
    title: str
    abbr: str = None
    weight: int = 0



from pydantic import BaseModel
from datetime import datetime
from .Works import Work


class ImportantWork(BaseModel):
    work: Work
    subjectName: str
    workTypeName: str


class Mark_(BaseModel):
    id: int
    id_str: str
    type: str
    value: str
    textValue: str
    person: int
    person_str: str
    work: int
    work_str: str
    lesson: int
    lesson_str: str
    number: int
    date: datetime
    workType: int
    mood: str
    use_avg_calc: bool


class MarksList(BaseModel):
    marks: list[Mark_]
    

class Mark(BaseModel):
    mark: Mark_


class Lesson(BaseModel):
    id: int
    date: datetime


class MarksCard(BaseModel):
    marks: list[Mark]
    lesson: Lesson
    isImportant: bool
    subjectName: str
    subjectId: int
    workTypeName: str


class Summary(BaseModel):
    importantWorks: list[ImportantWork] = []
    marksCards: list[MarksCard] = []
    dayEmotion: str
    feedMode: str


class NextDayHomeworks(BaseModel):
    work: Work
    subjectName: str
    workTypeName: str


class ImportantWorkType(BaseModel):
    id: int
    name: str


class NextDaySchedule(BaseModel):
    lessonId: int
    lessonStatus: str
    number: int
    subjectName: str
    importantWorkTypes: list[ImportantWorkType]


class TodayHomework(BaseModel):
    work: Work
    subjectName: str
    workTypeName: str


class TodaySchedule(BaseModel):
    lessonId: int
    lessonStatus: str
    number: int
    subjectName: str
    importantWorkTypes: list[ImportantWorkType]


class LessonLogEntry(BaseModel):
    person: int
    lesson: int
    person_str: str
    lesson_str: str
    comment: str
    status: str
    createdDate: datetime


class LogEntries(BaseModel):
    lessonLogEntry: LessonLogEntry
    subjectName: str
    lessonTitle: str


class FeedDay(BaseModel):
    """..."""
    date: datetime
    nextWorkingDayDate: datetime | None = None
    summary: Summary | None = None
    nextDayHomeworks: list[NextDayHomeworks] = []
    nextDaySchedule: list[NextDaySchedule] = []
    todayHomeworks: list[TodayHomework] = []
    todaySchedule: list[TodaySchedule] = []
    logEntries: list[LogEntries] = []



class UserFeed(BaseModel):
    """[GET] /v2.0/users/me/feed\n~~~\nЛента пользователя\n~~~\nПрава доступа: EducationalInfo\n~~~"""
    
    days: list[FeedDay] | None = None



from pydantic import BaseModel

from .EduGroups import EduGroup


class Context_Children(BaseModel):
    personId: int
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    schoolIds: list[int]
    groupIds: list[int]


class Context_school(BaseModel):
    id: int
    name: str
    type: str
    groupIds: list[int]
 

class Context(BaseModel):
    """[GET] /v2.0/users/me/context | /v2.0/users/{userID}/context\n~~~\nПолучение контекстной информации по пользователю\n~~~\nПрава доступа: CommonInfo, FriendsAndRelatives, EducationalInfo\n~~~\n"""
    
    userId: int
    avatarUrl: str
    roles: list[str]
    children: list[Context_Children]
    schools: list[Context_school]
    eduGroups: list[EduGroup]
    splitId: str
    personId: int
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    schoolIds: list[int]
    groupIds: list[int]


from pydantic import BaseModel


class School(BaseModel):
    """[GET] /v2.0/schools/{school}\n~~~\nПрофиль школы\n~~~"""
    
    fullName: str = None
    avatarSmall: str = None
    city: str = None
    municipality: str = None
    regionid: int = None
    markType: str = None
    timeZone: int = None
    uses_avg: bool = None
    uses_weighted_avg: bool = None
    id: int = None
    id_str: str = None
    name: str
    educationType: str
    tsoCityId: int = None
    tsoRegionTreePath: str = None

class PersonSchool(BaseModel):
    """[GET] /v2.0/schools/person-schools\n~~~\nСписок образовательных организаций пользователя\n~~~"""
    
    id: int
    id_str: str
    name: str
    educationType: str
    tsoCityId: int
    tsoRegionTreePath: str

class SchoolsCities(BaseModel):
    """[GET] /v2.0/schools/cities\n~~~\nСписок населенных пунктов, образовательные организации которых включены в Систему\n~~~"""
    
    id: int
    id_str: str
    name: str
    regionId: int
    regionId_str: str


from pydantic import BaseModel


class Children(BaseModel):
    """[GET] /v2.0/user/{userID}/children \n~~~\nПолучение списка детей по идентификатору родительского пользователя\n~~~\nПрава доступа: EducationalInfo\n~~~\n"""
    
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    sex: str


from pydantic import BaseModel


class UserRelatives_person(BaseModel):
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    sex: str


class UserRelatives(BaseModel):
    """[GET]\n~~~\n/v2.0/users/{user}/relatives\n~~~\n/v2.0/users/me/relatives\n~~~\n/v2.0/users/me/childrenrelatives\n~~~\nПолучение всех родственных связей (детей) произвольного пользователя.\n~~~"""
    
    type: str = None
    person: UserRelatives_person
    relatives: "list[UserRelatives]" = None



from datetime import datetime
from pydantic import BaseModel

class ReportingPeriod(BaseModel):
    """[GET] /v2.0/edu-groups/{eduGroup}/reporting-periods\n~~~\nСписок отчётных периодов для класса или учебной группы (Данный класс - 1 отчетный период)\n~~~"""
    
    id: int
    id_str: str
    start: datetime
    finish: datetime
    number: int
    type: str
    name: str
    year: int


class Holidays(BaseModel):
    id: int
    reportingPeriodGroupId: int
    dateStart: datetime
    dateFinish: datetime
    name: str
    description: str


class ReportingPeriodEduGroup(BaseModel):
    """[GET] /v2.0/edu-groups/{eduGroup}/reporting-periods-group\n~~~\nГруппа отчетных периодов для класса или учебной группы (Данный класс - 1 отчетный период)\n~~~"""
    
    id: int
    reportingPeriods: list[ReportingPeriod]
    holidays: list[Holidays]



from datetime import datetime
from pydantic import BaseModel

from .Subjects import Subject
from .Works import Work


class Lesson(BaseModel):
    """Урок"""
    
    id: int
    id_str: str = None
    title: str = None
    date: datetime = None
    number: int = None
    subject: Subject
    group: int
    status: str = None
    resultPlaceId: int = None
    works: list[Work] = []
    teachers: list[int] = []
    teachers_str: list[str] = []




from pydantic import BaseModel


class SchoolMemberships_school_school(BaseModel):
    fullName: str
    avatarSmall: str
    city: str
    municipality: str
    regionid: int
    markType: str
    timeZone: int
    uses_avg: bool
    uses_weighted_avg: bool
    id: int
    id_str: str
    name: str
    educationType: str
    tsoCityId: int
    tsoRegionTreePath: str


class SchoolMemberships_school_subject(BaseModel):
    id: int
    id_str: str
    name: str
    knowledgeArea: str
    fgosSubjectId: int


class SchoolMemberships_school_eduGroupMemberships_(BaseModel):
    id: int
    id_str: str
    parentIds: list[int]
    parentIds_str: list[str]
    type: str
    name: str
    fullName: str
    parallel: int
    timetable: int
    timetable_str: str
    status: str
    studyyear: int
    educationType: str
    subjects: list[SchoolMemberships_school_subject]
    journaltype: str
        

class SchoolMemberships_school_eduGroupMemberships(BaseModel):
    eduGroup: SchoolMemberships_school_eduGroupMemberships_


class SchoolMemberships_school(BaseModel):
    school: SchoolMemberships_school_school
    roles: list[str]
    editorRoles: list[str]
    eduGroupMemberships: list[SchoolMemberships_school_eduGroupMemberships]
    roles: list[str]
    subjects: list[SchoolMemberships_school_subject]


class SchoolMemberships(BaseModel):
    """[GET]\n~~~\n/v2.0/users/{user}/school-memberships\n~~~\n/v2.0/users/{user}/education\n~~~\n/v2.0/users/me/school-memberships\n~~~\n/v2.0/persons/{person}/school-memberships\n~~~\nСписок участий в школах для того или иного пользователя\n~~~\nПрава доступа: EducationalInfo\n~~~\n"""
    
    person: int | None = None
    person_str: str | None = None
    schools: list[SchoolMemberships_school]


from pydantic import BaseModel
from datetime import datetime

from .LessonLog import LessonLogEntries
from .Lessons import Lesson
from .Subjects import Subject
from .Works import Work
from .Marks import Mark


class criteriabasedmark(BaseModel):
    id: int
    id_str: str = None
    personid: int = None
    section: int = None
    value: int = None
    date: datetime = None


class WORKTYPE(BaseModel):
    id: int
    schoolId: int = None
    abbreviation: str = None
    name: str = None
    isFinal: bool = None
    isImportant: bool = None
    kindId: int = None
    kind: str = None


class RecentMarks(BaseModel):
    """[GET] /v2.0/persons/{person}/group/{group}/recentmarks\n~~~\nПоследние оценки/отметки посещаемости персоны по предмету указанному в параметре subject начиная с даты определенном в параметре fromDate и с ограничением на выводимое количество указанном в параметре limit\n~~~"""
    
    marks: list[Mark] = []
    criteriabasedmarks: list[criteriabasedmark] = []
    works: list[Work] = []
    subjects: list[Subject] = []
    workTypes: list[WORKTYPE] = []
    lessons: list[Lesson] = []
    lessonLogEntries: list[LessonLogEntries] = []


from .Authorities import Organization
from .Authorizations import TokenWithCode
from .Children import Children
from .Context import Context
from .EducationMemberships import SchoolMemberships
from .EduGroups import EduGroup
from .FinalMarks import EduGroupFinalMark
from .Homeworks import HomeWork
from .LessonLog import LessonLogEntries
from .Lessons import Lesson
from .MarkHistograms import MarksHistogram, MarksHistogramByPeriod
from .Marks import Mark
from .Persons import Person
from .RecentMarks import RecentMarks
from .ReportingPeriods import ReportingPeriod, ReportingPeriodEduGroup
from .Schedules import Schedule
from .Schools import School, SchoolsCities, PersonSchool
from .SchoolsParameters import SchoolParameters
from .Subjects import Subject
from .Tasks import Task
from .Teacher import SchoolTeacher, EduGroupTeacher, TeacherStudent
from .Timetables import TimeTable
from .UserFeeds import UserFeed
from .UserRelatives import UserRelatives
from .Users import User
from .WeightedAverageMarks import WeightedAverageMarks
from .Works import Work, EditStatusHomeWork
from .WorkTypes import WorkType


__all__ = [
    "Organization",
    "TokenWithCode",
    "Children",
    "Context",
    "SchoolMemberships",
    "EduGroup",
    "EduGroupFinalMark",
    "HomeWork",
    "LessonLogEntries",
    "Lesson",
    "MarksHistogram",
    "MarksHistogramByPeriod",
    "Mark",
    "Person", "RecentMarks",
    "ReportingPeriod", "ReportingPeriodEduGroup",
    "Schedule",
    "SchoolsCities",
    "PersonSchool",
    "School",
    "SchoolParameters",
    "Subject",
    "Task",
    "TeacherStudent",
    "SchoolTeacher",
    "EduGroupTeacher",
    "TimeTable",
    "UserFeed",
    "UserRelatives",
    "User",
    "WeightedAverageMarks",
    "Work",
    "EditStatusHomeWork",
    "WorkType"
]


from pydantic import BaseModel

class HobbyGroup(BaseModel):
    schoolId: int
    schoolId_str: str
    hobbyGroupName: str
    hobbyGroupDescription: str
    hobbyGroupSchedule: str
    hobbyGroupPayment: str = None
    hobbyGroupEnrollment: str


class LearningResult(BaseModel):
    schoolId: int
    schoolId_str: str
    studyYear: int
    prizesAtAllRussiaOlympiadPercentage: int = None
    basicCertificatesReceivedPercentage: int = None
    certificatesReceivedPercentage: int = None
    egeMathBaseLevelAverageScore: int = None
    egeMathProfileLevelAverageScore: int = None
    egeRussianAverageScore: int = None
    giaMathPassedPercentage: int = None
    giaMathPassedPerfectlyPercentage: int = None
    giaRussianPassedPercentage: int = None
    giaRussianPassedPerfectlyPercentage: int = None
    medalistsCount: int = None
    medalistsPercentage: int = None
    enrolledPercentage: int = None
    rankedInTheTop100: bool = None


class SchoolParameters(BaseModel):
    """[GET] /v2.0/schools/{school}/parameters\n~~~\nПараметры общеобразовательных организаций\n~~~"""
    
    schoolId: int
    schoolId_str: str
    municipality: str
    fullName: str
    shortName: str
    status: str
    foundationYear: int
    hasspecialty: bool
    specialty: str
    address: list[str]
    website: str
    email: str
    schoolPhone: str
    schoolDaysCount: int
    workHours: str
    shiftsCount: int
    eduOrgForm: str
    directorFullName: str
    directorPhone: str
    directorQualification: str
    directorPhoto: str
    licence: str
    accreditation: str
    charter: str
    cooperationName: str
    cooperationWebsite: str
    prescriptions: str
    activeJournalUrl: str
    inn: str
    facadePhoto: str
    cantinaPhoto: str
    gymPhoto: str
    classroomPhoto: str
    physicsroomPhoto: str
    chemicsryroomPhoto: str
    biologyroomPhoto: str
    mathroomPhoto: str
    russianroomPhoto: str
    musicroomPhoto: str
    educationalLevels: list[str]
    generalEducationalProgrammsLicense: str
    furtherEducationalProgramsLicense: str
    primarySchoolEducationalPrograms: list[str]
    hasAdaptedEducationalPrograms: bool = None
    hasOwnEducationalPrograms: bool = None
    ownEducationalPrograms: str
    hasDistEduTech: bool = None
    teachingForeignLanguages: list[str]
    studyPlan: str
    actualOccupancy: int = None
    designCapacity: int = None
    enrollmentIsOpen: bool = None
    vacantPlacesCount: int = None
    disabledChildrenCount: int = None
    firstStageGroupsCount: int = None
    secondStageGroupsCount: int = None
    thirdStageGroupsCount: int = None
    firstStageStudentsCount: int = None
    secondStageStudentsCount: int = None
    thirdStageStudentsCount: int = None
    teachersCount: int = None
    teachersWithHigherEducationCount: int = None
    highestCategoryTeachersCount: int = None
    publications: str
    laureatesOfContestsCount: str
    laureatesOfContestsInfo: str
    hasEducationalPsychologists: bool = None
    hasDefectologists: bool = None
    hasSpeechTherapists: bool = None
    hasSocialWorkers: bool = None
    hasMedicalWorkers: bool = None
    studentsPerComputer: int = None
    hasCompensatingClasses: bool = None
    hasBarrierFreeEnvironment: bool = None
    hasSwimmingPool: bool = None
    hasMedicalOffice: bool = None
    hasDiningRoom: bool = None
    hasGym: bool = None
    gymCount: int = None
    hasAssemblyHall: bool = None
    hasWinterGarden: bool = None
    hasCctv: bool = None
    hasGpd: bool = None
    schoolSiteArea: int = None
    hasEstheticZone: bool = None
    trainingExperimentalPlotArea: int = None
    hasSportArea: bool = None
    hasRecreationArea: bool = None
    hasZonesForTrainingInPreventionOfChildRoadTrafficInjuries: bool = None
    hasAreasOfEconomicPurpose: bool = None
    hobbyGroups: list[HobbyGroup]
    learningResults: list[LearningResult]



from pydantic import BaseModel

class markNumbers_mark(BaseModel):
    value: str
    count: int


class markNumbers(BaseModel):
    markNumber: int
    marks: list[markNumbers_mark]


class MarksHistogram(BaseModel):
    """Деперсонализированная гистограма оценок всего класса"""


    markNumbers: list[markNumbers]


class WORK(BaseModel):
    workId: int
    workId_str: str
    markNumbers: list[markNumbers]


class MarksHistogramByPeriod(BaseModel):
    """Деперсонализированная гистограма оценок всего класса за отчетный период"""
    
    works: list[WORK]


from pydantic import BaseModel

class Organization(BaseModel):
    """[GET] /v2.0/users/me/organizations/{organizationId}\n~~~\nДанные указанной организации пользователя\n~~~\nПрава доступа: EducationalInfo\n~~~\n"""
    
    OrganizationId: int
    OrganizationType: int
    RegionId: int
    DistrictId: int
    CityId: int


from datetime import datetime
from pydantic import BaseModel


class Task(BaseModel):
    """[GET] /v2.0/tasks/{task}\n~~~\n/v2.0/tasks\n~~~\n/v2.0/lessons/{lesson}/tasks\n~~~\n/v2.0/works/{work}/tasks\n~~~\n/v2.0/persons/{person}/tasks\n~~~\n/v2.0/tasks/{personId}/undone\n~~~\nДомашнее задание (Данный класс - 1 ДЗ)\n~~~\nПрава доступа: EducationalInfo\n~~~"""
    
    id: int
    id_str: str
    person: int
    person_str: str
    work: int
    work_str: str
    status: str
    targetDate: datetime



from datetime import date, datetime
import requests

from pydantic import BaseModel
from ..exceptions import raise_error, all_error_types_str, all_error_types_str_, APIError, all_error_types_str__dict


class BaseAPI:
    """
    Basic class for sync use schools.school.mosreg.ru API
    ``(api.school.mosreg.ru)``
    """
    
    def __init__(self, login: str = None, password: str = None, token: str = None) -> None:
        self.login = login
        self.password = password
        self.token = token
        self.session = requests.Session()
        
        self.API = "https://api.school.mosreg.ru/v2.0/"
        
        self.UserAgentDict = {"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:108.0) Gecko/20100101 Firefox/108.0"}
        
        if self.token is None:
            if self.login is None:
                raise ValueError("Token and login is None. Please add to arguments ``login=LOGIN, password=PASSWORD`` or ``token=TOKEN``")
            elif self.password is None:
                raise ValueError("Token and password is None. Please add to arguments ``login=LOGIN, password=PASSWORD`` or ``token=TOKEN``")
            else:
                self.token = self.get_token()
    
    @staticmethod
    def datetime_to_string(time: datetime | date = datetime.now()) -> str:
        """Сконвертировать datetime.datetime объект в строку(``str``) для использования в URL (METHOD)\n~~~"""
        return f"{time.year}-{time.month}-{time.day}T{time.hour}:{time.minute}:{time.second}" if isinstance(time, datetime) else f"{time.year}-{time.month}-{time.day}"
    
    @staticmethod
    def date_to_string(date: date = date.today()) -> str:
        """Сконвертировать datetime.date объект в строку(``str``) для использования в URL (METHOD)\n~~~"""
        return f"{date.year}-{date.month}-{date.day}"
    
    @staticmethod
    def get_headers(token: str):
        return {"Access-token": token}
    
    
    @property
    def headers(self):
        return {"Access-token": self.token}
    
    
    def get_token(self, params: dict = None):
        """
        Get Token with login&password
        
        :param:params - dict -> {"ReturnUrl": "...", "login": "...", "password": "..."}
        """
        
        params = params or {
            "ReturnUrl": "https://login.school.mosreg.ru/oauth2?response_type=token&client_id=bafe713c96a342b194d040392cadf82b&scope=EducationalInfo,CommonInfo,FriendsAndRelatives,SocialInfo,ContactInfo",
            "login": self.login, "password": self.password
        }
        URL = "https://login.school.mosreg.ru/login/"
        
        resp = self.session.post(URL, params=params, headers=self.UserAgentDict)
        
        if "result=success#access_token=" not in resp.url:
            resp = self.session.get(params["ReturnUrl"], headers=self.UserAgentDict)
            
            if "result=success#access_token=" not in resp.url:
                raise APIError(resp.url, resp.status_code, "Что-то не так...")

        if resp.status_code != 200:
            raise APIError(resp.url, resp.status_code,
                "Сайт лежит или ведутся технические работы, использование api временно невозможно"
            )
        
        return resp.url.replace("https://login.school.mosreg.ru/oauth2/Authorization/Result?response_type=token&client_id=bafe713c-96a3-42b1-94d0-40392cadf82b&scope=EducationalInfo,CommonInfo,FriendsAndRelatives,SocialInfo,ContactInfo&is_granted=False&result=success#access_token=", "").replace("&state=", "")
    
    
    def ParseListModels(self, model: BaseModel, text: str) -> list[BaseModel]:
        class ListModels(BaseModel):
            listik: list[model]
        
        return ListModels.parse_raw('{"listik": '+ text.replace("'", '"') + '}').listik
    
    
    @staticmethod
    def _check_response(response: requests.Response):
        if response.headers.get("Content-Type") == "text/html":
            error_html = response.content.decode()
            if "502" in error_html:
                raise_error(url=response.url, status_code=response.status_code, error_type="HTMLError", description=error_html)
            
            error_text = " ".join(
                word
                for word in error_html.split('<div class="error__description">')[-1]
                .split("<p>")[1]
                .strip()[:-4]
                .split()
            )
            raise_error(url=response.url, status_code=response.status_code, error_type="HTMLError", description=error_text)
        
        json_response = response.json()

        if isinstance(json_response, dict):
            if json_response.get("type") in all_error_types_str or json_response.get("type") in all_error_types_str_ or json_response.get("type") in all_error_types_str__dict.keys():
                raise_error(url=response.url, status_code=response.status_code, error_type=json_response.get("type"), description=json_response.get("description", None))
    
    
    def get(self, method: str, headers: dict = None, model: BaseModel | None = None, is_list: bool = False, return_json: bool = False, return_raw_text: bool = False, **kwargs):
        headers = headers or self.headers
        if headers.get("User-Agent", None) is None:
            headers.update({"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:108.0) Gecko/20100101 Firefox/108.0"})
        
        response = self.session.get(self.API + method, headers=headers, **kwargs)
        
        self._check_response(response)
        RAW_TEXT = response.text
        
        if return_json:
            return response.json()
        elif return_raw_text:
            return RAW_TEXT
        elif is_list:
            return self.ParseListModels(model, (RAW_TEXT))
        else:
            return model.parse_raw((RAW_TEXT))
    
    def post(self, method: str, headers: dict = None, json = None, data = None, model: BaseModel | None = None, is_list: bool = False, return_json: bool = False, return_raw_text: bool = False, **kwargs):
        headers = headers or self.headers
        if headers.get("User-Agent", None) is None:
            headers.update({"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:108.0) Gecko/20100101 Firefox/108.0"})
        
        response = self.session.post(self.API + method, headers=headers, json=json, data=data, **kwargs)
        
        self._check_response(response)
        RAW_TEXT = response.text()
        
        if return_json:
            return response.json()
        elif return_raw_text:
            return RAW_TEXT
        elif is_list:
            return self.ParseListModels(model, (RAW_TEXT))
        else:
            return model.parse_raw((RAW_TEXT))

    def put(self, method: str, headers: dict = None, json = None, data = None, model: BaseModel | None = None, is_list: bool = False, return_json: bool = False, return_raw_text: bool = False, **kwargs):
        headers = headers or self.headers
        if headers.get("User-Agent", None) is None:
            headers.update({"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:108.0) Gecko/20100101 Firefox/108.0"})
        
        response = self.session.put(self.API + method, headers=headers, json=json, data=data, **kwargs)
        
        self._check_response(response)
        RAW_TEXT = response.text()
        
        if return_json:
            return response.json()
        elif return_raw_text:
            return RAW_TEXT
        elif is_list:
            return self.ParseListModels(model, (RAW_TEXT))
        else:
            return model.parse_raw((RAW_TEXT))
    
    def delete(self, method: str, headers: dict = None, json = None, data = None, model: BaseModel | None = None, is_list: bool = False, return_json: bool = False, return_raw_text: bool = False, **kwargs):
        headers = headers or self.headers
        if headers.get("User-Agent", None) is None:
            headers.update({"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:108.0) Gecko/20100101 Firefox/108.0"})
        
        response = self.session.delete(self.API + method, headers=headers, json=json, data=data, **kwargs)
        
        self._check_response(response)
        RAW_TEXT = response.text()
        
        if return_json:
            return response.json()
        elif return_raw_text:
            return RAW_TEXT
        elif is_list:
            return self.ParseListModels(model, (RAW_TEXT))
        else:
            return model.parse_raw((RAW_TEXT))



from pydantic import BaseModel


class Person(BaseModel):
    """Персона."""
    
    id: int
    id_str: str
    userId: int
    userId_str: str
    firstName: str
    lastName: str
    middleName: str
    shortName: str
    sex: str